import inspect
import typing
from apistar import Component
from apistar.http import Response
from sqlalchemy.engine import Engine
from sqlalchemy.orm import sessionmaker, scoped_session

DBSession = scoped_session(sessionmaker())

Session = typing.TypeVar("Session")


class SQLAlchemySession(Component):
    def __init__(self, engine=None):
        if not isinstance(engine, Engine):
            raise ValueError('SQLAlchemySession must be instantiated with a sqlalchemy.engine.Engine object')
        self.engine = engine
        DBSession.configure(bind=self.engine)

    def resolve(self):
        return DBSession()

    def can_handle_parameter(self, parameter: inspect.Parameter) -> bool:
        return parameter.annotation is Session


class SQLAlchemyHook:
    def on_request(self, session: Session):
        return

    def on_response(self, session: Session, response: Response):
        DBSession.remove()
        return response

    def on_error(self, session: Session, response: Response):
        session.rollback()
        DBSession.remove()
        return response
